==> color_swatch-1.0.0.tar.gz: color_swatch/src/ColorSwatchManager.php <==
<?php

namespace Drupal\color_swatch;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ThemeExtensionList;
use Drupal\Core\Theme\ActiveTheme;
use Drupal\Core\Theme\ThemeManagerInterface;

/**
 * Class ColorSwatchManager
 *
 * @package Drupal\color_swatch
 */
class ColorSwatchManager {

  /**
   * Theme Extension List.
   *
   * @var \Drupal\Core\Extension\ThemeExtensionList
   */
  private $themeExtensionList;

  /**
   * Module Handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  private $moduleHandler;

  /**
   * Theme Manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  private $themeManager;

  public function __construct(ThemeManagerInterface $themeManager, ThemeExtensionList $themeExtensionList, ModuleHandlerInterface $moduleHandler) {
    $this->themeManager = $themeManager;
    $this->themeExtensionList = $themeExtensionList;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * Get theme color swatch placeholders.
   *
   * @param string $theme
   *
   * @return array
   */
  public function getThemeColorSwatchPlaceholdersInfo(string $theme): array {
    $info = $this->themeExtensionList->getExtensionInfo($theme);
    return $info['color-swatch']['placeholders'] ?? [];
  }

  /**
   * Get available theme color swatches
   *
   * @param string $theme
   *
   * @return array
   */
  public function getThemeColorSwatchesInfo(string $theme): array {
    $info = $this->themeExtensionList->getExtensionInfo($theme);
    return $info['color-swatch']['swatches'] ?? [];
  }

  /**
   * Get theme color swatch info of specific color swatch.
   *
   * @param string $theme
   * @param string $swatchName
   *
   * @return array
   */
  public function getThemeColorSwatchInfo(string $theme, string $swatchName): array {
    $swatches = $this->getThemeColorSwatchesInfo($theme);
    return $swatches[$swatchName] ?? $this->getColorSwatchThemeSettings($theme)['settings'][$swatchName] ?? [];
  }

  /**
   * Get the default color swatch information.
   *
   * @param string $theme
   *
   * @return string
   */
  public function getThemeDefaultColorSwatchName(string $theme): string {
    $info = $this->themeExtensionList->getExtensionInfo($theme);
    return $info['color-swatch']['default'] ?? '';
  }

  /**
   * Convert an RGB hex to HSL values.
   *
   * @param string $hex
   *
   * @return array|float[]|int[]
   */
  public function hexToHsl(string $hex): array {
    [$red, $green, $blue] = $this->hexToRgbTriplet($hex);

    $min = min($red, min($green, $blue));
    $max = max($red, max($green, $blue));
    $delta = $max - $min;
    $lightness = ($min + $max) / 2;
    $saturation = 0;

    if ($lightness > 0 && $lightness < 1) {
      $saturation = $delta / ($lightness < 0.5 ? (2 * $lightness) : (2 - 2 * $lightness));
    }

    $hue = 0;
    if ($delta <= 0) {
      return $this->buildHslArray($hue, $saturation, $lightness);
    }

    if ($max === $red && $max !== $green) {
      $hue += ($green - $blue) / $delta;
    }
    if ($max === $green && $max !== $blue) {
      $hue += (2 + ($blue - $red) / $delta);
    }
    if ($max === $blue && $max !== $red) {
      $hue += (4 + ($red - $green) / $delta);
    }
    $hue /= 6;

    return $this->buildHslArray($hue, $saturation, $lightness);
  }

  /**
   * Converts a hex color into an RGB triplet.
   *
   * @param string $hex
   * @param bool $normalize
   *
   * @return mixed
   */
  public function hexToRgbTriplet(string $hex, bool $normalize = FALSE) {
    $hex = substr($hex, 1);
    if (strlen($hex) === 3) {
      $hex = $hex[0] . $hex[0] . $hex[1] . $hex[1] . $hex[2] . $hex[2];
    }
    $c = hexdec($hex);
    $rgb = [];
    for ($i = 16; $i >= 0; $i -= 8) {
      $rgb[] = (($c >> $i) & 0xFF) / ($normalize ? 255 : 1);
    }

    return $rgb;
  }

  /**
   * Get a color swatch.
   *
   * @param string $theme
   * @param string $swatchName
   *
   * @return \Drupal\color_swatch\ColorSwatch
   */
  public function getColorSwatch(string $theme, string $swatchName): ColorSwatch {
    return new ColorSwatch($this, $theme, $swatchName);
  }

  /**
   * Get current active theme.
   *
   * @return \Drupal\Core\Theme\ActiveTheme
   */
  public function getActiveTheme(): ActiveTheme {
    return $this->themeManager->getActiveTheme();
  }

  /**
   * Get color swatch theme settings.
   *
   * @param string $theme
   *
   * @return array|mixed|null
   */
  public function getColorSwatchThemeSettings(string $theme) {
    return theme_get_setting('color_swatch', $theme);
  }

  /**
   * Check if the theme has color swatches.
   *
   * @param string $theme
   *
   * @return bool
   */
  public function hasColorSwatches(string $theme): bool {
    return !empty($this->getThemeDefaultColorSwatchName($theme)) &&
      !empty($this->getThemeColorSwatchPlaceholdersInfo($theme)) &&
      !empty($this->getThemeColorSwatchesInfo($theme));
  }

  /**
   * Get the color swatch that is active.
   *
   * @param string $theme
   *
   * @return \Drupal\color_swatch\ColorSwatch
   */
  public function getActiveColorSwatch(string $theme): ColorSwatch {
    $swatchName = $this->getActiveColorSwatchName($theme);
    $colorSwatch = $this->getColorSwatch($theme, $swatchName);
    $this->moduleHandler->alter('active_color_swatch', $colorSwatch);

    return $colorSwatch;
  }

  /**
   * Get active color swatch name.
   *
   * @param string $theme
   *
   * @return string
   */
  public function getActiveColorSwatchName(string $theme): string {
    $themeSettings = $this->getColorSwatchThemeSettings($theme);
    $swatchName = $this->getThemeDefaultColorSwatchName($theme);
    if (isset($themeSettings['active']) && !empty($this->getThemeColorSwatchInfo($theme, $swatchName))) {
      $swatchName = $themeSettings['active'];
    }

    $this->moduleHandler->alter('active_color_swatch_name', $colorSwatch);

    return $swatchName;
  }

  /**
   * Get the module handler.
   *
   * @return \Drupal\Core\Extension\ModuleHandlerInterface
   */
  public function getModuleHandler(): ModuleHandlerInterface {
    return $this->moduleHandler;
  }

  /**
   * Build the hsl array.
   *
   * @param int|float $hue
   * @param int|float $saturation
   * @param int|float $lightness
   *
   * @return array
   */
  private function buildHslArray($hue, $saturation, $lightness): array {
    return ['h' => $hue, 's' => $saturation, 'l' => $lightness];
  }

}

==> color_swatch-1.0.1.tar.gz: color_swatch/src/ColorSwatchManager.php <==
<?php

namespace Drupal\color_swatch;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ThemeExtensionList;
use Drupal\Core\Theme\ActiveTheme;
use Drupal\Core\Theme\ThemeManagerInterface;

/**
 * Class ColorSwatchManager
 *
 * @package Drupal\color_swatch
 */
class ColorSwatchManager {

  /**
   * Theme Extension List.
   *
   * @var \Drupal\Core\Extension\ThemeExtensionList
   */
  private $themeExtensionList;

  /**
   * Module Handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  private $moduleHandler;

  /**
   * Theme Manager.
   *
   * @var \Drupal\Core\Theme\ThemeManagerInterface
   */
  private $themeManager;

  public function __construct(ThemeManagerInterface $themeManager, ThemeExtensionList $themeExtensionList, ModuleHandlerInterface $moduleHandler) {
    $this->themeManager = $themeManager;
    $this->themeExtensionList = $themeExtensionList;
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * Get theme color swatch placeholders.
   *
   * @param string $theme
   *
   * @return array
   */
  public function getThemeColorSwatchPlaceholdersInfo(string $theme): array {
    $info = $this->themeExtensionList->getExtensionInfo($theme);
    return $info['color-swatch']['placeholders'] ?? [];
  }

  /**
   * Get available theme color swatches
   *
   * @param string $theme
   *
   * @return array
   */
  public function getThemeColorSwatchesInfo(string $theme): array {
    $info = $this->themeExtensionList->getExtensionInfo($theme);
    return $info['color-swatch']['swatches'] ?? [];
  }

  /**
   * Get theme color swatch info of specific color swatch.
   *
   * @param string $theme
   * @param string $swatchName
   *
   * @return array
   */
  public function getThemeColorSwatchInfo(string $theme, string $swatchName): array {
    $swatches = $this->getThemeColorSwatchesInfo($theme);
    return $swatches[$swatchName] ?? $this->getColorSwatchThemeSettings($theme)['settings'][$swatchName] ?? [];
  }

  /**
   * Get the default color swatch information.
   *
   * @param string $theme
   *
   * @return string
   */
  public function getThemeDefaultColorSwatchName(string $theme): string {
    $info = $this->themeExtensionList->getExtensionInfo($theme);
    return $info['color-swatch']['default'] ?? '';
  }

  /**
   * Validate if the input of the hex is in the format #ffffff, #fff, ffffff or fff
   *
   * @param string $hex
   *
   * @return string
   */
  public function validateHex(string $hex): string {
    // Complete patterns like #ffffff or #fff
    if (preg_match('/^#([0-9a-fA-F]{6})$/', $hex) || preg_match('/^#([0-9a-fA-F]{3})$/', $hex)) {
      // Remove #
      $hex = substr($hex, 1);
    }

    // Complete patterns without # like ffffff or 000000
    if (preg_match('/^([0-9a-fA-F]{6})$/', $hex)) {
      return $hex;
    }

    // Short patterns without # like fff or 000
    if (preg_match('/^([0-9a-f]{3})$/', $hex)) {
      // Spread to 6 digits
      return $hex[0] . $hex[0] . $hex[1] . $hex[1] . $hex[2] . $hex[2];
    }

    // If input value is invalid return black
    return '000000';
  }

  /**
   * Get HSL values from rgb hex.
   *
   * @param string $hex
   *
   * @return array
   */
  public function hexToHsl(string $hex): array {
    //Validate Hex Input
    $hex = $this->validateHex($hex);

    // Split input by color
    $hexArray = str_split($hex, 2);

    // Convert color values to value between 0 and 1
    $r = (hexdec($hexArray[0])) / 255;
    $g = (hexdec($hexArray[1])) / 255;
    $b = (hexdec($hexArray[2])) / 255;

    // Determine lowest & highest value and chroma
    $max = max($r, $g, $b);
    $min = min($r, $g, $b);
    $chroma = $max - $min;

    // Calculate Luminosity
    $l = ($max + $min) / 2;

    // If chroma is 0, the given color is grey
    // therefore hue and saturation are set to 0
    if ($chroma === 0) {
      $h = 0;
      $s = 0;
      return $this->buildHslArray($h, $s, $l);
    }

    $h_ = 0;
    if ($max === $r) {
      $h_ = fmod((($g - $b) / $chroma), 6);
      if ($h_ < 0) {
        $h_ = (6 - fmod(abs($h_), 6));
      }
    }

    if ($max === $g) {
      $h_ = ($b - $r) / $chroma + 2;
    }

    if ($max === $b) {
      $h_ = ($r - $g) / $chroma + 4;
    }

    $h = $h_ / 6;
    $s = 1 - abs(2 * $l - 1);

    return $this->buildHslArray($h, $s, $l);
  }

  /**
   * Get a color swatch.
   *
   * @param string $theme
   * @param string $swatchName
   *
   * @return \Drupal\color_swatch\ColorSwatch
   */
  public function getColorSwatch(string $theme, string $swatchName): ColorSwatch {
    return new ColorSwatch($this, $theme, $swatchName);
  }

  /**
   * Get current active theme.
   *
   * @return \Drupal\Core\Theme\ActiveTheme
   */
  public function getActiveTheme(): ActiveTheme {
    return $this->themeManager->getActiveTheme();
  }

  /**
   * Get color swatch theme settings.
   *
   * @param string $theme
   *
   * @return array|mixed|null
   */
  public function getColorSwatchThemeSettings(string $theme) {
    return theme_get_setting('color_swatch', $theme);
  }

  /**
   * Check if the theme has color swatches.
   *
   * @param string $theme
   *
   * @return bool
   */
  public function hasColorSwatches(string $theme): bool {
    return !empty($this->getThemeDefaultColorSwatchName($theme)) &&
      !empty($this->getThemeColorSwatchPlaceholdersInfo($theme)) &&
      !empty($this->getThemeColorSwatchesInfo($theme));
  }

  /**
   * Get the color swatch that is active.
   *
   * @param string $theme
   *
   * @return \Drupal\color_swatch\ColorSwatch
   */
  public function getActiveColorSwatch(string $theme): ColorSwatch {
    $swatchName = $this->getActiveColorSwatchName($theme);
    $colorSwatch = $this->getColorSwatch($theme, $swatchName);
    $this->moduleHandler->alter('active_color_swatch', $colorSwatch);

    return $colorSwatch;
  }

  /**
   * Get active color swatch name.
   *
   * @param string $theme
   *
   * @return string
   */
  public function getActiveColorSwatchName(string $theme): string {
    $themeSettings = $this->getColorSwatchThemeSettings($theme);
    $swatchName = $this->getThemeDefaultColorSwatchName($theme);
    if (isset($themeSettings['active']) && !empty($this->getThemeColorSwatchInfo($theme, $swatchName))) {
      $swatchName = $themeSettings['active'];
    }

    $this->moduleHandler->alter('active_color_swatch_name', $colorSwatch);

    return $swatchName;
  }

  /**
   * Get the module handler.
   *
   * @return \Drupal\Core\Extension\ModuleHandlerInterface
   */
  public function getModuleHandler(): ModuleHandlerInterface {
    return $this->moduleHandler;
  }

  /**
   * Build the hsl array.
   *
   * @param int|float $hue
   * @param int|float $saturation
   * @param int|float $lightness
   *
   * @return array
   */
  private function buildHslArray($hue, $saturation, $lightness): array {
    return ['h' => $hue, 's' => $saturation, 'l' => $lightness];
  }

}
